@@ -1,7 +1,7 @@
 package Crypt::SecretBuffer;
 # VERSION
 # ABSTRACT: Prevent accidentally leaking a string of sensitive data
-$Crypt::SecretBuffer::VERSION = '0.006';
+$Crypt::SecretBuffer::VERSION = '0.007';
 
 use strict;
 use warnings;
@@ -16,15 +16,17 @@ bootstrap Crypt::SecretBuffer;
 
 {
    package Crypt::SecretBuffer::Exports;
-$Crypt::SecretBuffer::Exports::VERSION = '0.006';
+$Crypt::SecretBuffer::Exports::VERSION = '0.007';
 use Exporter 'import';
-   @Crypt::SecretBuffer::Exports::EXPORT_OK= qw( secret_buffer secret NONBLOCK AT_LEAST );
+   @Crypt::SecretBuffer::Exports::EXPORT_OK= qw( secret_buffer secret unmask_secrets_to
+      NONBLOCK AT_LEAST
+   );
    sub secret_buffer {
       Crypt::SecretBuffer->new(@_)
    }
-   *secret= *secret_buffer;
-   *NONBLOCK=  *Crypt::SecretBuffer::NONBLOCK;
-   *AT_LEAST=  *Crypt::SecretBuffer::AT_LEAST;
+   *secret=   *secret_buffer;
+   *NONBLOCK= *Crypt::SecretBuffer::NONBLOCK;
+   *AT_LEAST= *Crypt::SecretBuffer::AT_LEAST;
 }
 
 sub import {
@@ -98,6 +100,11 @@ Crypt::SecretBuffer - Prevent accidentally leaking a string of sensitive data
   
   undef $buf;             # no copies of password remain in memory.
 
+  # pass secret directly to a XS function without copying a scalar
+  use Crypt::SecretBuffer 'unmask_secrets_to';
+  unmask_secrets_to(\&c_function, $buf);
+  $buf->unmask_to(\&c_function)
+
 =head1 DESCRIPTION
 
 This module helps you protect a secret value from getting copied around unintentionally or
@@ -135,11 +142,84 @@ stringification of the buffer reveals the secret or not.  For instance:
   }
   say $buf;                            # stringifies as [REDACTED]
 
+or:
+
+  $buf->unmask_to(\&some_xs_function);
+
 There is no guarantee that the XS function in that example wouldn't make a copy of your secret,
 but this at least provides the secret buffer directly to the XS code that calls C<SvPV> without
 making a copy.  If an XS module is aware of Crypt::SecretBuffer, it can use a more official
 L</C API> that doesn't rely on perl stringification behavior.
 
+=head2 Supporting SecretBuffer Without a Dependency
+
+If you have a module where you'd like to optionally receive secrets via SecretBuffer objects,
+but don't want your module to depend on Crypt::SecretBuffer, here are some useful recipes:
+
+  # unmask a single variable with duck-typing
+  sub connect($self, $dsn, $user, $password) {
+    local $password->{stringify_mask}= undef
+      if blessed $password && $password->can('stringify_mask');
+    my $dbh= DBI->connect($dsn, $user, $password);
+    ...
+  }
+
+  # use unmask_secrets_to with a fallback if SecretBuffer is not installed
+  BEGIN {
+    eval 'use Crypt::SecretBuffer qw/unmask_secrets_to/; 1'
+    or eval 'sub unmask_secrets_to { shift->(@_) }'
+  }
+  sub connect($self, $dsn, $user, $password) {
+    my $dbh= unmask_secrets_to \&DBI::connect, $dsn, $user, $password;
+    ...
+  }
+
+  /*
+   * In C code, perform the 'local' technique and overloaded stringification
+   */
+  const char *actual_pass= NULL;
+  STRLEN actual_pass_len;
+  if (sv_isobject(password) && sv_derived_from(password, "Crypt::SecretBuffer")) {
+    HV *hv= (HV*) SvRV(password);
+    SV **svp= hv_fetchs(hv, "stringify_mask", 0);
+    if (svp) {
+      SAVESPTR(*svp);
+      *svp= &PL_sv_undef;
+    } else {
+      hv_stores(hv, "stringify_mask", newSVsv(&PL_sv_undef));
+      SAVEDELETE(hv, savepv("stringify_mask"), 14);
+    }
+  }
+  actual_pass= SvPV(password, actual_pass_len);
+
+  /*
+   * In C code, conditionally access the SecretBuffer C API
+   */
+  typedef struct {
+    char *data;
+    size_t len, capacity;
+    SV *stringify_sv;
+  } secret_buffer;
+  typedef secret_buffer* sb_from_magic_t(SV *ref, int flags);
+  
+  ...
+     const char *actual_pass= NULL;
+     STRLEN actual_pass_len;
+  
+     HV *secretbuffer_api = get_hv("Crypt::SecretBuffer::C_API", 0);
+     if (secretbuffer_api) { /* only becomes true after 'use Crypt::SecretBuffer;' */
+       SV **svp = hv_fetch(secretbuffer_api, "secret_buffer_from_magic", 24, 0);
+       sb_from_magic_t *sb_from_magic= svp && *svp? (sb_from_magic_t*) SvIV(*svp) : NULL;
+       secret_buffer *buf;
+       if (sb_from_magic && (buf= sb_from_magic(password, 0))) {
+         actual_pass= buf->data;
+         actual_pass_len= buf->len;
+       }
+     }
+     if (!actual_pass)
+       actual_pass= SvPV(password, actual_pass_len);
+  ...
+
 =head1 CONSTRUCTORS
 
 =head2 new
@@ -221,6 +301,25 @@ return the string C<< "[REDACTED]" >> or whatever custom string you store in C<s
 If you set C<stringify_mask> to C<undef>, it exposes the secret.  You can use C<local> to limit
 the scope of this exposure.
 
+=head2 unmask_to
+
+  $buf->unmask_to(\&coderef);
+
+Pass the secret value as an argument to a code-ref.  If you want to prevent the secret from
+leaking into perl's heap, the coderef should be an XS function.  If you pass it to a perl
+function and load the parameter into a local variable, then it leaks into perl's heap.
+You *might* be OK if you pass it to a perl function and leave it in C<< @_ >> until passing it
+to an XS function.
+
+This is equivalent to, but more efficient than
+
+  {
+    local $buf->{stringify_mask}= undef;
+    coderef->($buf);
+  }
+
+See also: L</unmask_secrets_to>.
+
 =head2 index
 
   $ofs= $buf->index($str);
@@ -344,6 +443,13 @@ Shorthand function for calling L</new>.
 
 Shorthand function for calling L</new>.
 
+=item unmask_secrets_to
+
+  unmask_secrets_to \&coderef, $arg1, $arg2, ...;
+
+Call a coderef with a list of arguments, and any argument which is a SecretBuffer will be
+replaced by a scalar referencing the actual secret.
+
 =back
 
 =head1 C API
@@ -389,7 +495,7 @@ instructions how to report security vulnerabilities.
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 AUTHOR
 
